/*******************************************************************************
Copyright (C) Marvell International Ltd. and its affiliates

********************************************************************************
Marvell GPL License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File in accordance with the terms and conditions of the General 
Public License Version 2, June 1991 (the "GPL License"), a copy of which is 
available along with the File in the license.txt file or by writing to the Free 
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 or 
on the worldwide web at http://www.gnu.org/licenses/gpl.txt. 

THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE IMPLIED 
WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY 
DISCLAIMED.  The GPL License provides additional details about this warranty 
disclaimer.

*******************************************************************************/

#include <config.h>
#include <common.h>
#include <command.h>
#include <pci.h>

#include "mvTypes.h"
#include "mvCtrlEnvLib.h"
#include "mvFlash.h"

#include "mvUart.h"
#include "mvDS1339.h"

#if defined(MV_88F5181)
#include "mvEthPhy.h"
#include "mvIdma.h"
#endif

#if defined(CFG_NAND_BOOT)
#include <linux/mtd/nand.h>

/* references to names in cmd_nand.c */
#define NANDRW_READ	0x01
#define NANDRW_WRITE	0x00
#define NANDRW_JFFS2	0x02
extern struct nand_chip nand_dev_desc[];
int nand_rw (struct nand_chip* nand, int cmd,
	    size_t start, size_t len,
	    size_t * retlen, u_char * buf);
int nand_erase(struct nand_chip* nand, size_t ofs,
				size_t len, int clean);

#endif /* defined(CFG_NAND_BOOT) */


#if defined BUFFALO_DEBUG
 #define TRACE(x)	x
#else
 #define TRACE(x)
#endif

#if defined(CONFIG_BUFFALO_PLATFORM)
 #include <environment.h>
 #include "mvGpp.h"

 //#define BUFFALO_ENABLE_HDD_POWER_CONTROL
 //#define BUFFALO_ENABLE_USB_POWER_CONTROL
 //#define BUFFALO_ENABLE_FAN_CONTROL
 //#define BUFFALO_ENABLE_LED_CONTROL
 //#define BUFFALO_ENABLE_BUZZER_CONTROL
 //#define BUFFALO_ENABLE_BOARD_INFO
 //#define BUFFALO_ENABLE_PHY_TEST
 //#define BUFFALO_ENABLE_ENVSWITCH
 //#define BUFFALO_ENABLE_MICON_TEST
 //#define BUFFALO_ENABLE_REG_TEST
 //#define BUFFALO_ENABLE_PEX_TEST
 //#define BUFFALO_ENABLE_GPIO_TEST
 #define BUFFALO_STORE_MACADDR
 //#define BUFFALO_DEBUG

 extern env_t *env_ptr;
 extern uchar environment[];
 extern uchar default_environment[];
#endif

static unsigned int flash_in_which_sec(flash_info_t *fl,unsigned int offset)
{
	unsigned int sec_num;
	if(NULL == fl)
		return 0xFFFFFFFF;

	for( sec_num = 0; sec_num < fl->sector_count ; sec_num++){
		/* If last sector*/
		if (sec_num == fl->sector_count -1)
		{
			if((offset >= fl->start[sec_num] - fl->start[0]) && 
			   (offset < fl->size) )
			{
				return sec_num;
			}

		}
		else
		{
			if((offset >= fl->start[sec_num] - fl->start[0]) && 
			   (offset < fl->start[sec_num + 1] - fl->start[0]) )
			{
				return sec_num;
			}

		}
	}
	/* return illegal sector Number */
	return 0xFFFFFFFF;

}

/*******************************************************************************
burn a u-boot.bin on the Boot Flash
********************************************************************************/
extern flash_info_t flash_info[];       /* info for FLASH chips */
#include <net.h>

#if defined(CFG_NAND_BOOT)
/* Boot from NAND flash */
/* Write u-boot image into the nand flash */
int nand_burn_uboot_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int filesize;
	MV_U32 total, ret = 0;
	extern char console_buffer[];


	load_addr = 0x100000; 
	if(argc == 2) {
		copy_filename (BootFile, argv[1], sizeof(BootFile));
	}
	else { 
		copy_filename (BootFile, "u-boot.bin", sizeof(BootFile));
		printf("using default file \"u-boot.bin\" \n");
	}
 
	if ((filesize = NetLoop(TFTP)) < 0)
		return 0;

	printf("Override Env parameters? (y/n)");
	readline(" ");
	if( strcmp(console_buffer,"Y") == 0 || 
	    strcmp(console_buffer,"yes") == 0 ||
	    strcmp(console_buffer,"y") == 0 ) {

		printf("Erase Env parameters sector %d... ",CFG_ENV_OFFSET);
		nand_erase(nand_dev_desc + 0, CFG_ENV_OFFSET, CFG_ENV_SECT_SIZE, 0);
		printf("\n");
	}

	printf("Erase %d - %d ... ",CFG_MONITOR_BASE, CFG_MONITOR_LEN + CFG_ENV_SECT_SIZE);
	nand_erase(nand_dev_desc + 0, CFG_MONITOR_BASE, CFG_MONITOR_LEN + CFG_ENV_SECT_SIZE, 0);
	
	printf("\nCopy to Nand Flash... ");
	ret = nand_rw(nand_dev_desc + 0,
				  NANDRW_WRITE | NANDRW_JFFS2, CFG_MONITOR_BASE, CFG_MONITOR_LEN + CFG_ENV_SECT_SIZE,
			      &total, (u_char*)0x100000);

  	if (ret || total != CFG_MONITOR_LEN + CFG_ENV_SECT_SIZE)
		return 1;

	printf("\ndone\n");	

	return 1;
}

U_BOOT_CMD(
        bubt,      2,     1,      nand_burn_uboot_cmd,
        "bubt	- Burn an image on the Boot Nand Flash.\n",
        " file-name \n"
        "\tBurn a binary image on the Boot Nand Flash, default file-name is u-boot.bin .\n"
);

/* Write nboot loader image into the nand flash */
int nand_burn_nboot_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int filesize;
	MV_U32 total, ret = 0;
	extern char console_buffer[];


	load_addr = 0x100000; 
	if(argc == 2) {
		copy_filename (BootFile, argv[1], sizeof(BootFile));
	}
	else { 
		copy_filename (BootFile, "nboot.bin", sizeof(BootFile));
		printf("using default file \"nboot.bin\" \n");
	}
 
	if ((filesize = NetLoop(TFTP)) < 0)
		return 0;
 
	printf("Erase %d - %d ... ",CFG_NBOOT_BASE, CFG_NBOOT_LEN);
	nand_erase(nand_dev_desc + 0, CFG_NBOOT_BASE, CFG_NBOOT_LEN , 0);
	
	printf("\nCopy to Nand Flash... ");
	ret = nand_rw(nand_dev_desc + 0,
				  NANDRW_WRITE | NANDRW_JFFS2, CFG_NBOOT_BASE, CFG_NBOOT_LEN,
			      &total, (u_char*)0x100000);

  	if (ret || total != CFG_NBOOT_LEN)
		return 1;

	printf("\ndone\n");	

	return 1;
}

U_BOOT_CMD(
        nbubt,      2,     1,      nand_burn_nboot_cmd,
        "nbubt	- Burn a boot loader image on the Boot Nand Flash.\n",
        " file-name \n"
        "\tBurn a binary boot loader image on the Boot Nand Flash, default file-name is nboot.bin .\n"
);

#else
/* Boot from Nor flash */
int nor_burn_uboot_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int filesize;
	MV_U32 s_first,s_end,env_sec;
	extern char console_buffer[];


	s_first = flash_in_which_sec(&flash_info[0],
								CFG_MONITOR_BASE - flash_info[0].start[0]);
	s_end = flash_in_which_sec(&flash_info[0],
								CFG_MONITOR_BASE + CFG_MONITOR_LEN -1 - flash_info[0].start[0]);

	env_sec = flash_in_which_sec(&flash_info[0],
								CFG_ENV_ADDR - flash_info[0].start[0]);


	load_addr = 0x100000; 
	if(argc == 2) {
		copy_filename (BootFile, argv[1], sizeof(BootFile));
	}
	else { 
		copy_filename (BootFile, "u-boot.bin", sizeof(BootFile));
		printf("using default file \"u-boot.bin\" \n");
	}
 
	if ((filesize = NetLoop(TFTP)) < 0)
		return 0;
 
	printf("Un-Protect Flash Monitor space\n");
	flash_protect (FLAG_PROTECT_CLEAR,
		       CFG_MONITOR_BASE,
		       CFG_MONITOR_BASE + CFG_MONITOR_LEN - 1,
		       &flash_info[0]);

	printf("Override Env parameters? (y/n)");
	readline(" ");
	if( strcmp(console_buffer,"Y") == 0 || 
	    strcmp(console_buffer,"yes") == 0 ||
	    strcmp(console_buffer,"y") == 0 ) {

		flash_protect (FLAG_PROTECT_CLEAR,
				   flash_info[0].start[env_sec],
				   flash_info[0].start[env_sec] + CFG_ENV_SECT_SIZE - 1,
				   &flash_info[0]);

		printf("Erase Env parameters sector %d... ",env_sec);
		flash_erase(&flash_info[0], env_sec, env_sec);

		flash_protect (FLAG_PROTECT_SET,
				   flash_info[0].start[env_sec],
				   flash_info[0].start[env_sec] + CFG_ENV_SECT_SIZE - 1,
				   &flash_info[0]);

	}

	printf("Erase %d - %d sectors... ",s_first,s_end);
	flash_erase(&flash_info[0], s_first, s_end);

	printf("Copy to Flash... ");

	flash_write ( (uchar *)0x100000 + CFG_MONITOR_IMAGE_OFFSET,
				  CFG_MONITOR_BASE,
				  filesize - CFG_MONITOR_IMAGE_OFFSET);

	printf("done\nProtect Flash Monitor space\n");
	flash_protect (FLAG_PROTECT_SET,
		       CFG_MONITOR_BASE,
		       CFG_MONITOR_BASE + CFG_MONITOR_LEN - 1,
		       &flash_info[0]);

	return 1;
}

U_BOOT_CMD(
        bubt,      2,     1,      nor_burn_uboot_cmd,
        "bubt	- Burn an image on the Boot Flash.\n",
        " file-name \n"
        "\tBurn a binary image on the Boot Flash, default file-name is u-boot.bin .\n"
);
#endif /* defined(CFG_NAND_BOOT) */

/*saito change*/
#if defined(CONFIG_BUFFALO_PLATFORM)

#define BUFFALO_CHECK_HAVE_A_MICON(x)		if(IsSupportMicon() == 0) { printf("\nSorry, this board does not support this command\n\n"); return x; }
#define BUFFALO_CHECK_NOT_HAVE_A_MICON(x)	if(IsSupportMicon() == 1) { printf("\nSorry, this board does not support this command\n\n"); return x;}

#include "mvGpp.h"

#if defined(BUFFALO_ENABLE_BOARD_INFO)
MV_U32
do_show_board_info(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	printf("board configuration\n");
	printf("\tVGPP_CONFIG_NUM=%4d\n", VGPP_CONFIG_NUM);
	printf("\tBoardId        =%4d\t(0x%04x)\n", mvBoardIdGet(), mvBoardIdGet());
	printf("\tgBoardId       =%4d\t(0x%04x)\n", gBoardId, gBoardId);
	printf("\n");

	printf("power control bit\n");
	if(PIN_NUM_HDD_POWER != 255)
		printf("\thdd_power      =%4d\t(0x%04x)\n", PIN_NUM_HDD_POWER, BIT_HDD_POWER);
	else
		printf("\thdd_power      = N_A\n");
	if(PIN_NUM_HDD_POWER1 != 255)
		printf("\thdd_power1     =%4d\t(0x%04x)\n", PIN_NUM_HDD_POWER1, BIT_HDD_POWER1);
	else
		printf("\thdd_power1     = N_A\n");
	if(PIN_NUM_USB_POWER != 255)
		printf("\tusb_power      =%4d\t(0x%04x)\n", PIN_NUM_USB_POWER, BIT_USB_POWER);
	else
		printf("\tusb_power      = N_A\n");
	if(PIN_NUM_USB_POWER2 != 255)
		printf("\tusb_power2     =%4d\t(0x%04x)\n", PIN_NUM_USB_POWER2, BIT_USB_POWER2);
	else
		printf("\tusb_power2     = N_A\n");
	if(PIN_NUM_USB_POWER3 != 255)
		printf("\tusb_power3     =%4d\t(0x%04x)\n", PIN_NUM_USB_POWER3, BIT_USB_POWER3);
	else
		printf("\tusb_power3     = N_A\n");
	if(PIN_NUM_USB_POWER4 != 255)
		printf("\tusb_power4     =%4d\t(0x%04x)\n", PIN_NUM_USB_POWER4, BIT_USB_POWER4);
	else
		printf("\tusb_power4     = N_A\n");
	printf("\n");

	printf("led control bit\n");
	if(PIN_NUM_LED_INFO != 255)
		printf("\tled_info       =%4d\t(0x%04x)\n", PIN_NUM_LED_INFO, BIT_LED_INFO);
	else
		printf("\tled_info       = N_A\n");
	if(PIN_NUM_LED_POWER != 255)
		printf("\tled_power      =%4d\t(0x%04x)\n", PIN_NUM_LED_POWER, BIT_LED_POWER);
	else
		printf("\tled_power      = N_A\n");
	if(PIN_NUM_LED_ALARM != 255)
		printf("\tled_alarm      =%4d\t(0x%04x)\n", PIN_NUM_LED_ALARM, BIT_LED_ALARM);
	else
		printf("\tled_alarm      = N_A\n");
	if(PIN_NUM_LED_FUNC != 255)
		printf("\tled_func       =%4d\t(0x%04x)\n", PIN_NUM_LED_FUNC, BIT_LED_FUNC);
	else
		printf("\tled_func       = N_A\n");
	if(PIN_NUM_LED_WLAN != 255)
		printf("\tled_wlan       =%4d\t(0x%04x)\n", PIN_NUM_LED_WLAN, BIT_LED_WLAN);
	else
		printf("\tled_wlan       = N_A\n");
	if(PIN_NUM_LED_SECURE != 255)
		printf("\tled_secure     =%4d\t(0x%04x)\n", PIN_NUM_LED_SECURE, BIT_LED_SECURE);
	else
		printf("\tled_secure     = N_A\n");
	if(PIN_NUM_LED_AOSS != 255)
		printf("\tled_aoss       =%4d\t(0x%04x)\n", PIN_NUM_LED_AOSS, BIT_LED_AOSS);
	else
		printf("\tled_aoss       = N_A\n");
	printf("\n");

	printf("fan control bit\n");
	if(PIN_NUM_FAN_FULL != 255)
		printf("\tfan_full       =%4d\t(0x%04x)\n", PIN_NUM_FAN_FULL, BIT_FAN_FULL);
	else
		printf("\tfan_full       = N_A\n");
	if(PIN_NUM_FAN_LOCK != 255)
		printf("\tfan_lock       =%4d\t(0x%04x)\n", PIN_NUM_FAN_LOCK, BIT_FAN_LOCK);
	else
		printf("\tfan_lock       = N_A\n");
	if(PIN_NUM_FAN_LOW != 255)
		printf("\tfan_low        =%4d\t(0x%04x)\n", PIN_NUM_FAN_LOW, BIT_FAN_LOW);
	else
		printf("\tfan_low        = N_A\n");
	if(PIN_NUM_FAN_LOCK != 255)
		printf("\tfan_high       =%4d\t(0x%04x)\n", PIN_NUM_FAN_HIGH, BIT_FAN_HIGH);
	else
		printf("\tfan_high       = N_A\n");
	printf("\n");

	printf("sw control bit\n");
	if(PIN_NUM_SW_INIT != 255)
		printf("\tsw_init        =%4d\t(0x%04x)\n", PIN_NUM_SW_INIT, BIT_SW_INIT);
	else
		printf("\tsw_init        = N_A\n");
	if(PIN_NUM_SW_POWER != 255)
		printf("\tsw_power       =%4d\t(0x%04x)\n", PIN_NUM_SW_POWER, BIT_SW_POWER);
	else
		printf("\tsw_power       = N_A\n");
	if(PIN_NUM_SW_RAID != 255)
		printf("\tsw_raid        =%4d\t(0x%04x)\n", PIN_NUM_SW_RAID, BIT_SW_RAID);
	else
		printf("\tsw_raid        = N_A\n");
	if(PIN_NUM_SW_AUTO_POWER != 255)
		printf("\tsw_auto_power  =%4d\t(0x%04x)\n", PIN_NUM_SW_AUTO_POWER, BIT_SW_AUTO_POWER);
	else
		printf("\tsw_auto_power  = N_A\n");
	if(PIN_NUM_SW_AOSS != 255)
		printf("\tsw_aoss        =%4d\t(0x%04x)\n", PIN_NUM_SW_AOSS, BIT_SW_AOSS);
	else
		printf("\tsw_aoss        = N_A\n");
	printf("\n");
}

U_BOOT_CMD(
        board_info,      2,     1,     do_show_board_info,
        "board_info   -show board information.\n",
	"\n"
	"\t Show board information. like boardid, gpp pin function...\n"
);
#endif

#if defined(BUFFALO_ENABLE_GPIO_TEST)
MV_U32
get_sw_status(MV_VOID)
{
	return mvGppValueGet(0, ((BIT_SW_INIT) | (BIT_SW_POWER) | (BIT_SW_AUTO_POWER) | (BIT_SW_FUNC)));
}

MV_VOID
do_get_sw_status(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_NOT_HAVE_A_MICON();

	MV_U32 sw_status;
	sw_status = get_sw_status();
	switch(argc){
	case 2:
		if(strcmp(argv[1], "power") == 0){
			if(sw_status & BIT_SW_POWER)
				printf("power sw is pushed\n");
			else
				printf("power sw is released\n");
			return;
		}else if(strcmp(argv[1], "init") == 0){
			if(sw_status & BIT_SW_INIT)
				printf("init sw is pushed\n");
			else
				printf("init sw is released\n");
			return;
		}else if(strcmp(argv[1], "auto") == 0){
			if(sw_status & BIT_SW_AUTO_POWER)
				printf("auto sw is pushed\n");
			else
				printf("auto sw is released\n");
			return ;
		}else if(strcmp(argv[1], "func") == 0){
			if(sw_status & BIT_SW_FUNC)
				printf("func sw is pushed\n");
			else
				printf("func sw is released\n");
		}else{
			printf("sw_status = 0x%04x\n", sw_status);
			return ;
		}
		break;
	default:
		printf("Usage:\n%s\n", cmdtp->usage);
		printf("sw_status = 0x%04x\n", sw_status);
		break;
	}
}

U_BOOT_CMD(
        get_sw,      2,     1,     do_get_sw_status,
        "get_sw\t\t- without micon. get switch status.\n",
	"\n"
	"\t Get sw status. follow options are supported [power|init|auto|func]\n"
);
#endif

MV_VOID
mdelay(MV_U32 wait_msec)
{
	MV_U32 msec = 0;

	for(msec=0; msec<wait_msec; msec++){
		udelay(1000);
	}
}

MV_VOID
buffalo_led_code_indicate(MV_U8 indicate_type, MV_U8 value)
{
	BUFFALO_CHECK_NOT_HAVE_A_MICON();

	MV_U8	led_10 = 0, led_1 = 0;
	MV_U8	led_10_lp = 0, led_1_lp = 0;
	int lp;
	
#if defined BUFFALO_SUPPORT_VIRTUALIZED_GPP
	if(indicate_type == BIT_LED_ALARM)
	{
		buffalo_gpp_info_led_off();
	}
	else if(indicate_type == BIT_LED_INFO)
	{
		buffalo_gpp_alarm_led_off();
	}
	else
	{
		return;
	}
#else
	switch(indicate_type){
	case BIT_LED_ALARM:
		buffalo_info_led_off();
		break;
	case BIT_LED_INFO:
		buffalo_alarm_led_off();
		break;
	default:
		return ;
	}
#endif

	buffalo_gpp_hdd_power_off();
	buffalo_gpp_power_led_off();

	led_10 = value / 10;
	led_1 = value % 10;

	while(1)
	{
		buffalo_gpp_led_off(indicate_type);
		mdelay(1000);
		for(led_10_lp=0; led_10_lp < led_10; led_10_lp++){
			buffalo_gpp_led_on(indicate_type);
			mdelay(1000);
			buffalo_gpp_led_off(indicate_type);
			mdelay(300);
			// switch handle.
			if(buffalo_gpp_sw_is_init_pushed() ||
			   buffalo_gpp_sw_is_func_pushed())
				break;
		}

		for(led_1_lp=0; led_1_lp < led_1; led_1_lp++){
			buffalo_gpp_led_on(indicate_type);
			mdelay(500);
			buffalo_gpp_led_off(indicate_type);
			mdelay(300);
			// switch handle.
			if(buffalo_gpp_sw_is_init_pushed() ||
			   buffalo_gpp_sw_is_func_pushed())
				break;
		}
		// switch handle.

		if(buffalo_gpp_sw_is_init_pushed() ||
		   buffalo_gpp_sw_is_func_pushed())
			break;
			
	} // end of while

	printf("\nfunc switch pushed, and exited loop\n");
	printf("wait until func switch released.\n");
	while (1)
	{
		// wait until func switch release.
		if(!buffalo_gpp_sw_is_func_pushed())
			break;
	}
}

#if defined(BUFFALO_ENABLE_LED_CONTROL)
MV_VOID
buffalo_led_roulet(MV_VOID)
{
	MV_U32 step = 0;
	MV_U32 manipulate_target=0;

	while(1)
	{
		buffalo_gpp_led_control(BIT_LED_POWER, BUFFALO_LED_FLAG_OFF);
		buffalo_gpp_led_control(BIT_LED_WLAN, BUFFALO_LED_FLAG_OFF);
		buffalo_gpp_led_control(BIT_LED_SECURE, BUFFALO_LED_FLAG_OFF);
		buffalo_gpp_led_control(BIT_LED_AOSS, BUFFALO_LED_FLAG_OFF);
		buffalo_gpp_led_control(BIT_LED_FUNC, BUFFALO_LED_FLAG_OFF);
		//buffalo_gpp_led_control(BIT_LED_INFO, BUFFALO_LED_FLAG_OFF);
		buffalo_gpp_led_control(BIT_LED_ALARM, BUFFALO_LED_FLAG_OFF);

		switch(step)
		{
		case 0:
			manipulate_target = BIT_LED_POWER;
			break;
		case 10:
			manipulate_target = BIT_LED_WLAN;
			break;
		case 20:
			manipulate_target = BIT_LED_SECURE;
			break;
		case 30:
			manipulate_target = BIT_LED_AOSS;
			break;
		case 40:
			manipulate_target = BIT_LED_FUNC;
			break;
		case 50:
			manipulate_target = BIT_LED_INFO;
			manipulate_target = BIT_LED_ALARM;
			break;
		default:
			break;
		}
		buffalo_gpp_led_control(manipulate_target, BUFFALO_LED_FLAG_ON);
		mdelay(100);
		if(step >= 50)
			step = 0;
		else
			step+=10;
	}
}

MV_VOID
do_led_control(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_NOT_HAVE_A_MICON();

	MV_U32 manipulate_target=0;

	switch(argc){
	case 3:
		if(strcmp(argv[1], "power") == 0){
			manipulate_target = BIT_LED_POWER;
		}else if(strcmp(argv[1], "info") == 0){
			manipulate_target = BIT_LED_INFO;
		}else if(strcmp(argv[1], "alarm") == 0){
			manipulate_target = BIT_LED_ALARM;
		}else if(strcmp(argv[1], "func") == 0){
			manipulate_target = BIT_LED_FUNC;
		}else if(strcmp(argv[1], "wlan") == 0){
			manipulate_target = BIT_LED_WLAN;
		}else if(strcmp(argv[1], "secure") == 0){
			manipulate_target = BIT_LED_SECURE;
		}else if(strcmp(argv[1], "aoss") == 0){
			manipulate_target = BIT_LED_AOSS;
		}else if(strcmp(argv[1], "roulet") == 0){
			buffalo_led_roulet();
		}else{
			printf("Usage:\n%s\n", cmdtp->usage);
			return ;
		}

		if(strcmp(argv[2], "on") == 0){
			buffalo_gpp_led_control(manipulate_target, BUFFALO_LED_FLAG_ON);
		}else if(strcmp(argv[2], "off") == 0){
			buffalo_gpp_led_control(manipulate_target, BUFFALO_LED_FLAG_OFF);
		}else if(strcmp(argv[2], "blink") == 0){
			buffalo_gpp_led_control(manipulate_target, BUFFALO_LED_FLAG_BLINK_ON);
		}else{
			buffalo_led_code_indicate(manipulate_target, simple_strtoul(argv[2], NULL, 10));
			
			return ;
		}
		break;
	default:
		printf("Usage:\n%s\n", cmdtp->usage);
		return ;
	}
		
}

U_BOOT_CMD(
	led,         3,     1,     do_led_control,
	"led\t\t- without micon. call led control routine(for debug).\n",
	"\n"
	"\t led control ruotine. this is nomicon ruotine.\n"
	"\t follow options are supported. [power|info|alarm|func|wlan|secure|aoss] [on|off|blink]\n"
);
#endif

#if !defined(CONFIG_BUFFALO_LGL_PLATFORM)
void
do_shutdown(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();
	system_shutdown() ;
}

U_BOOT_CMD(
        shutdown,      1,     1,     do_shutdown,
        "shutdown\t- with micon. Poff .\n",
	"\n"
	"\t send shutdown command to micon\n"
);

void
do_bootend(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();
	system_bootend() ;
}


U_BOOT_CMD(
        bootend,      1,     1,     do_bootend,
        "bootend\t- with micon. send bootend.\n",
	"\n"
	"\t send bootend command to micon\n"
);
#endif

#if defined BUFFALO_ENABLE_ENVSWITCH
void
do_envswitch(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	if(argc != 2)
		return 1;

	if(strcmp(argv[1], "present") == 0){
		//env_ptr = (env_t *)(&environment[0]);
		env_ptr = (env_t *)CFG_ENV_ADDR;
	}else if(strcmp(argv[1], "default") == 0){
		env_ptr = (env_t *)(&default_environment[0]);
	}
}

U_BOOT_CMD(
        envsw,      2,     1,     do_envswitch,
        "envsw   - debug command\n",
);
#endif

#if defined BUFFALO_ENABLE_BUZZER_CONTROL
void
do_buzzer(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();

	switch(argc){
	case 2:
		if(strcmp(argv[1], "off") == 0){
			mcon_bz_off();
			return ;
		}else if(strcmp(argv[1], "power_on") == 0){
			mcon_bz_power_on();
			return ;
		}else if(strcmp(argv[1], "button") == 0){
			mcon_bz_button();
			return ;
		}else if(strcmp(argv[1], "em_on") == 0){
			mcon_bz_em_on();
			return ;
		}else if(strcmp(argv[1], "on") == 0){
			mcon_bz_on();
			return ;
		}else if(strcmp(argv[1], "on3_off3") == 0){
			mcon_bz_on3_off3();
			return ;
		}else if(strcmp(argv[1], "on5_off3") == 0){
			mcon_bz_on5_off3();
			return ;
		}
			
		break;
	default:
		break;
	}
}

U_BOOT_CMD(
	buzzer,		2,	1,	do_buzzer,
	"buzzer\t\t- with micon. buzzer command.\n",
	"\n"
	"\t led control micon mode\n"
	"\t supported options are below. [on | off | button | on3_off3 | on5_off3 | em_on | power_on]\n"
);
#endif

#if defined BUFFALO_ENABLE_LED_CONTROL
 #if !defined(CONFIG_BUFFALO_LGL_PLATFORM)
MV_U16
led_common_parser(char *pszName)
{
	BUFFALO_CHECK_HAVE_A_MICON();

	if(strcmp(pszName, "ac1") == 0)
		return BIT4;
	else if(strcmp(pszName, "ac2") == 0)
		return BIT5;
	else if(strcmp(pszName, "ac3") == 0)
		return BIT6;
	else if(strcmp(pszName, "ac4") == 0)
		return BIT7;
	else if(strcmp(pszName, "st1") == 0)
		return BIT8;
	else if(strcmp(pszName, "st2") == 0)
		return BIT9;
	else if(strcmp(pszName, "st3") == 0)
		return BIT10;
	else if(strcmp(pszName, "st4") == 0)
		return BIT11;
	else if(strcmp(pszName, "power") == 0)
		return BIT0;
	else if(strcmp(pszName, "info") == 0)
		return BIT1;
	else if(strcmp(pszName, "diag") == 0)
		return BIT2;
	else if(strcmp(pszName, "link") == 0)
		return BIT3;

	return 0;	
}

void
do_led_on_off(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();

	MV_U16 led_on_off_control_bit=0;
	int i=0;

	switch(argc){
	case 2:
		if(strcmp(argv[1], "off") == 0){
			led_on_off_control_bit=0;
			break;
		}else if(strcmp(argv[1], "all_on") == 0){
			led_on_off_control_bit=0xffff;
			break;
		}
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 13:
	case 14:
	case 15:
		for(i=1; i<argc; i++){
			led_on_off_control_bit |= led_common_parser(argv[i]);
		}
		break;
	default:
		printf("Usage:\n%s\n", cmdtp->usage);
		goto led_on_off_exit;
		break;
	}
	mcon_led_on_off_w(led_on_off_control_bit);

led_on_off_exit:
	printf("\nPresent led on off status=0x%04x\n", mcon_led_on_off_r());

}

U_BOOT_CMD(
	led_on_off,	16,	1,	do_led_on_off,
	"led_on_off\t- with micon. led on off control setting.\n",
	"\n"
	"\t led control routine with micon.\n"
	"\t supported options are below. [all_on | off | ac1 | ac2 | ac3 | ac4 | st1 | st2 | st3 | st4 | power | info | diag | link]\n"
);

void
do_led_blink(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();

	MV_U16 led_blink_control_bit=0;
	int i=0;

	switch(argc){
	case 2:
		if(strcmp(argv[1], "off") == 0){
			led_blink_control_bit=0;
			break;
		}else if(strcmp(argv[1], "all_on") == 0){
			led_blink_control_bit=0xffff;
			break;
		}
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 13:
	case 14:
	case 15:
		for(i=1; i<argc; i++){
			led_blink_control_bit |= led_common_parser(argv[i]);
		}
		break;
	default:
		printf("Usage:\n%s\n", cmdtp->usage);
		goto led_blink_exit;
		break;
	}
	mcon_led_blink_w(led_blink_control_bit);

led_blink_exit:
	printf("\nPresent led blink status=0x%04x\n", mcon_led_blink_r());

}

U_BOOT_CMD(
	led_blink,	16,	1,	do_led_blink,
	"led_blink\t- with micon. led blink control setting.\n",
	"\n"
	"\t led blink control routine with micon.\n"
	"\t supported options are below. [all_on | off | ac1 | ac2 | ac3 | ac4 | st1 | st2 | st3 | st4 | power | info | diag | link]\n"
);

void
do_led_cpu_mcon(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();

	MV_U16 led_cpu_mcon_control_bit=0;
	int i=0;

	switch(argc){
	case 2:
		if(strcmp(argv[1], "off") == 0){
			led_cpu_mcon_control_bit=0;
			break;
		}else if(strcmp(argv[1], "all_on") == 0){
			led_cpu_mcon_control_bit=0xffff;
			break;
		}
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 13:
	case 14:
	case 15:
		for(i=1; i<argc; i++){
			led_cpu_mcon_control_bit |= led_common_parser(argv[i]);
		}
		break;
	default:
		printf("Usage:\n%s\n", cmdtp->usage);
		goto led_control_exit;
		break;
	}
	mcon_led_cpu_mcon_w(led_cpu_mcon_control_bit);

led_control_exit:
	printf("\nPresent led control status=0x%04x\n", mcon_led_cpu_mcon_r());

}

U_BOOT_CMD(
	led_cpu_mcon,	16,	1,	do_led_cpu_mcon,
	"led_cpu_mcon\t- with micon. led cpu micon control setting.\n",
	"\n"
	"\t led cpu-mcon control routine with micon.\n"
	"\t supported options are below. [all_on | off | ac1 | ac2 | ac3 | ac4 | st1 | st2 | st3 | st4 | power | info | diag | link]\n"
);
 #endif // !CONFIG_BUFFALO_LGL_PLATFORM
#endif BUFFALO_ENABLE_LED_CONTROL

#if defined BUFFALO_ENABLE_FAN_CONTROL
MV_U8
do_fan_get_rpm(void)
{
	if(IsSupportMicon() == 1)
	{
		printf("fan rpm=%d\n", mcon_fan_get_rpm() * 10);
	}
	else if(IsSupportMicon() == 0)
	{
		if((MV_REG_READ(GPP_DATA_IN_REG(0)) & BIT_FAN_LOCK) == 0)
		{
			printf("fan is rotating\n");
		}else{
			printf("fan is stopped\n");
		}
	}
}

U_BOOT_CMD(
	fan_rpm,	1,	1,	do_fan_get_rpm,
	"fan_rpm\t\t- with micon. fan get rpm.\n",
	"\n"
	"\t Get fan rpm. need micon.\n"
);

MV_U8
do_fan_control_sub(char *pszcmd)
{
	BUFFALO_CHECK_HAVE_A_MICON();

	if(strcmp(pszcmd, "stop") == 0)
		return 0;
	else if(strcmp(pszcmd, "slow") == 0)
		return 1;
	else if(strcmp(pszcmd, "fast") == 0)
		return 2;
	else if(strcmp(pszcmd, "full") == 0)
		return 3;

	return -1;
}

void
do_fan_control(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();

	MV_U8 fan_control_bit=0xff;
	MV_U8 fan_speed=0;

	switch(argc){
	case 2:
		fan_control_bit = do_fan_control_sub(argv[1]);
		break;
	default:
		goto fan_control_exit;
		break;
	}

	mcon_fan_control_w(fan_control_bit);

fan_control_exit:
	printf("\nPresent fan control status = 0x%02x\n", mcon_fan_control_r());
}

U_BOOT_CMD(
	fan_control,	2,	1,	do_fan_control,
	"fan_control\t- with micon. fan control  .\n",
	"\n"
	"\t fan control. with micon.\n"
	"\t supported options are below [stop | slow | fast | full]\n"
);

#endif // end of BUFFALO_ENABLE_FAN_CONTROL

#if defined BUFFALO_ENABLE_HDD_POWER_CONTROL
MV_U8
do_hdd_power_sub(char *pszName)
{
	BUFFALO_CHECK_HAVE_A_MICON();

	if(strcmp(pszName, "hdd1") == 0)
		return BIT0;
	else if(strcmp(pszName, "hdd2") == 0)
		return BIT1;
	else if(strcmp(pszName, "hdd3") == 0)
		return BIT2;
	else if(strcmp(pszName, "hdd4") == 0)
		return BIT3;
	else if(strcmp(pszName, "hdd5") == 0)
		return BIT4;
	else if(strcmp(pszName, "hdd6") == 0)
		return BIT5;
	else if(strcmp(pszName, "hdd7") == 0)
		return BIT6;
	else if(strcmp(pszName, "hdd8") == 0)
		return BIT7;

	return 0;	
}

void
do_hdd_power(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	//BUFFALO_CHECK_HAVE_A_MICON();

	MV_U8 hdd_control_bit=0;
	int i=0;

	if(IsSupportMicon())
	{
		switch(argc){
		case 2:
			if(strcmp(argv[1], "off") == 0){
				hdd_control_bit=0;
				break;
			}else if(strcmp(argv[1], "all_on") == 0){
				hdd_control_bit=(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
				break;
			}
			// goto usual routine.
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
		case 8:
			for(i=1; i<(argc);i++){
				TRACE(printf("argv[%d]=%s\n", i, argv[i]));
				hdd_control_bit |= do_hdd_power_sub(argv[i]);
			}
			break;
		default:
			goto hdd_power_exit;
			break;
		}

		mcon_hdd_power_control_w(hdd_control_bit);

hdd_power_exit:
		printf("\nPresent hdd power status=0x%02x\n", mcon_hdd_power_control_r());
		TRACE(printf("hdd_control_bit=0x%02x\n", hdd_control_bit));
	}
	else
	{
		switch(argc){
		case 2:
			if(strcmp(argv[1], "off") == 0)
			{
				hdd_control_bit=BUFFALO_HDD_POWER_OFF;
				break;
			}
			else if(strcmp(argv[1], "on") == 0)
			{
				hdd_control_bit=BUFFALO_HDD_POWER_ON;
				break;
			}
			break;
		default:
			break;
		}
		buffalo_gpp_hdd_power_control((MV_U8)hdd_control_bit);
	}
}

U_BOOT_CMD(
	hdd_power,    9,     0,     do_hdd_power,
	"hdd_power\t- hdd power control  .\n",
	"\n"
	"\t hdd power contorl routine. with micon and without micon\n"
	"\t supported options with micon are below.    [all_on | off | hdd1 | hdd2 | hdd3 | hdd4 | hdd5 | hdd6 | hdd7 | hdd8]\n"
	"\t supported options without micon are below. [on | off]\n"
);
#endif //BUFFALO_ENABLE_HDD_POWER_CONTROL

#if defined BUFFALO_ENABLE_USB_POWER_CONTROL
void
do_usb_power(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	MV_U8 usb_control_bit=0;

	if(IsSupportMicon())
	{
		
	}
	else
	{
		switch(argc){
		case 2:
			if(strcmp(argv[1], "off") == 0)
			{
				usb_control_bit=BUFFALO_USB_POWER_OFF;
				break;
			}
			else if(strcmp(argv[1], "on") == 0)
			{
				usb_control_bit=BUFFALO_USB_POWER_ON;
				break;
			}
			break;
		default:
			break;
		}
		buffalo_gpp_usb_power_control((MV_U8)usb_control_bit);
	}

}

U_BOOT_CMD(
	usb_power,    2,     0,     do_usb_power,
	"usb_power\t- without micon. usb power control  .\n",
	"\n"
	"\t usb power contorl routine. without micon\n"
	"\t supported options are below. [on | off]\n"
);
#endif // end of BUFFALO_ENABLE_USB_POWER_CONTROL

#if defined(BUFFALO_ENABLE_MICON_TEST)
void do_micon (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();

	unsigned char	send_data[34];
	unsigned char	rcv_data[34];
	unsigned char   rcv_cmd;
	int loop;
	int i=0, j=0;
	int counter=0;

	memset(send_data, (char)NULL, sizeof(send_data));
	memset(rcv_data, (char)NULL, sizeof(rcv_data));
	if((argc > 35) || (argc < 2))
	{
		printf ("Usage:\n%s\n", cmdtp->usage);
	}
	for(loop = 0;loop < argc ;loop++)
	{
		send_data[loop] = (unsigned char)simple_strtoul(argv[loop + 1], NULL, 16);
	}
	if((send_data[0] & 0x7f) > 32)
	{
				printf ("Usage:\n%s\n", cmdtp->usage);
	}
	for(i=0;i<(argc - 1);i++){
		printf("%s > send_data[%d]=0x%02x\n", __FUNCTION__, i, send_data[i]);
	}

	if(micon_communication_reset()==0){
		printf("micon communication reset success!\n");
	}else{
		printf("micon communication reset failed\n");
	}

	while(counter < 3){
		send_micon_packet((argc - 1) ,&send_data);
 		rcv_micon_packet_ll(&rcv_data, &rcv_cmd);
		counter++;
		if(send_data[1]==rcv_cmd){
			break;
		}else{
			printf("Unexpected cmd received(0x%02x). try again ...\n", rcv_cmd);
		}
	}

#if defined BUFFALO_DEBUG
	for(i=0; i<34; i++){
		if((i % 8) == 0){
			if(i > 0){
				printf(" [");
				for(j=i-8; j<i; j++){
					if(j ==(i-4)){
						printf(" ");
					}

					if((rcv_data[j] > '!') & (rcv_data[j] < '}')){
						printf("%c", rcv_data[j]);
					}else{
						printf(".");
					}
				}
				printf("]");
			}
			printf("\n0x ");
		}
		printf("%02x ", rcv_data[i]);
	}
	printf("\n");
#endif

}

U_BOOT_CMD(
	micon,  34,  0,  do_micon,
	"micon\t\t- with micon. command(2byte hex) -data(witin 32byte hex) \n",
	"\n"
	"\t send command to micon\n"
);

MV_VOID
do_sw_power(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	BUFFALO_CHECK_HAVE_A_MICON();

	switch(argc){
	case 2:
		if(strcmp(argv[1], "on") == 0){
			buffalo_software_power_on();
		}else if(strcmp(argv[1], "off") == 0){
			buffalo_software_power_off();
		}
		break;
	default:
		printf ("Usage:\n%s\n", cmdtp->usage);
		break;
	}
}

U_BOOT_CMD(
	sw_power,	2,	1,	do_sw_power,
	"sw_power\t- with micon. software power control interface\n",
	"\n"
	"\t software power on. with micon\n"
);
#endif // BUFFALO_ENABLE_MICON_TEST

#if 0 /*for test*/ 
void do_test_func(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
ups_recover_set() ;
}

U_BOOT_CMD(
	test,  1,  1,  do_test_func,
	"test a function \n",
);
void do_test2_func(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
 rtc_reg_read() ;
}
U_BOOT_CMD(
	test2,  1,  1,  do_test2_func,
	"test2 a function \n",
);
#endif

MV_U32
buffaloAtoi(char *src)
{
	char *p = src;
	MV_U32 res = 0;
	MV_U32 coef = 10;

	if(strncmp(src, "0x", strlen("0x")) == 0)
	{
		coef = 0x10;
		p++;
		p++;
	}

	//printf("p=%s\n", p);
	while((*p >= '0' && *p <= '9') || (*p >= 'a' && *p <= 'f') || (*p >= 'A' && *p <= 'F')){
		//printf("p=%s, res=%d\n", p, res);
		res = res * coef;
		if(*p >= '0' && *p <= '9')
			res += (*p - '0');
		else if(*p >= 'a' && *p <= 'f')
			res += 0xa + (*p - 'a');
		else if(*p >= 'A' && *p <= 'F')
			res += 0xA + (*p - 'A');
		else
			return 0;
		p++;
	}
	return res;
}

void
buffaloEthRegPrint(MV_U16 dev_addr)
{
	int i = 0;
	MV_U16 tmp_reg = 0x00;

	printf("dev_addr=0x%02x\n", dev_addr);
	for(i = 0;i < 32; i++)
	{
		mvEthPhyRegRead(dev_addr, i, &tmp_reg);
		printf("0x%02x = 0x%04x\n", i, tmp_reg);
	}
}

#if defined(BUFFALO_ENABLE_PHY_TEST)
void
do_phy_test_func(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int i;
	MV_U16 dev_addr = 0x00;
	MV_U16 reg_addr = 0x00;
	MV_U16 tmp_reg = 0x00;

	switch(argc)
	{
	case 2:
		dev_addr = buffaloAtoi(argv[1]);
		printf("dev_addr=%d\n", dev_addr);
		if (dev_addr >= 0 && dev_addr <= 0x1c)
			buffaloEthRegPrint(dev_addr);
		else
			printf("error\n");
		break;
	case 3:
		dev_addr = buffaloAtoi(argv[1]);
		reg_addr = buffaloAtoi(argv[2]);

		if (dev_addr >= 0 && dev_addr <= 0x1c)
		{
			if(reg_addr >= 0 && reg_addr <= 0x1f)
			{
				mvEthPhyRegRead(dev_addr, reg_addr, &tmp_reg);
				printf("dev_addr=0x%02x\n", dev_addr);
				printf("0x%02x = 0x%04x\n", reg_addr, tmp_reg);
			}
		}
		else
			printf("error\n");
		break;
	default:
		for(i = 0; i <= 0x1c; i++)
		{
			buffaloEthRegPrint(i);
		}
		break;
	}
}

U_BOOT_CMD(
	phy,	3,	1,	do_phy_test_func,
	"phy\t- phy register check function\n",
);
#endif // end of BUFFALO_ENABLE_PHY_TEST

#if defined(BUFFALO_ENABLE_REG_TEST)
void
do_cpu_reg_func(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	MV_U32 reg_val = 0x0;
	MV_U32 reg_addr = 0x0;
	int i = 0;

	MV_U32 permitted_reg[] = {	MPP_CONTROL_REG0,
					MPP_CONTROL_REG1,
					MPP_CONTROL_REG2,
					DEV_MULTI_CONTROL,
					0x1500,
					0x1504,
					0x1508,
					0x150c,
					0x00,
				};

	switch(argc)
	{
	case 3:
		// write
		reg_addr = buffaloAtoi(argv[1]);
		for(i = 0;permitted_reg[i] != 0x00; i++)
		{
			if(reg_addr == permitted_reg[i])
			{
				reg_val = buffaloAtoi(argv[2]);
				MV_REG_WRITE(reg_addr, reg_val);
				return ;
			}
		}
		printf("not permitted to access 0x%08x\n", reg_addr);
		break;
	case 2:
		// read
		reg_addr = buffaloAtoi(argv[1]);
		for(i = 0;permitted_reg[i] != 0x00; i++)
		{
			printf("reg_addr=0x%08x, permitted_reg[%d]=0x%08x\n", reg_addr, i, permitted_reg[i]);
			if(reg_addr == permitted_reg[i])
			{
				reg_val = MV_REG_READ(reg_addr);
				printf("0x%08x = 0x%08x\n", reg_addr, reg_val);
				return ;
			}
		}
		printf("not permitted to access 0x%08x\n", reg_addr);
		break;
	default:
		for(i = 0;permitted_reg[i] != 0x00; i++)
		{
			reg_val = MV_REG_READ(permitted_reg[i]);
			printf("0x%08x = 0x%08x\n", permitted_reg[i], reg_val);
		}
		break;
	}

}

U_BOOT_CMD(
	cpu_reg,	3,	1,	do_cpu_reg_func,
	"cpu_reg\t- cpu register test function.\n",
);

#endif

#if defined(BUFFALO_ENABLE_PEX_TEST)
void
do_pex_reg_func(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	#include "mvPexRegs.h"
	int i = 0;
	MV_U32 reg_addr = 0x00;
	MV_U32 reg_val = 0x00;
	MV_U32 permitted_reg[]={	PEX_BAR_CTRL_REG(0,0),
					PEX_BAR_CTRL_REG(0,1),
					PEX_BAR_CTRL_REG(0,2),
					PEX_EXP_ROM_BAR_CTRL_REG(0),
					PEX_CFG_ADDR_REG(0),
					PEX_CFG_DATA_REG(0),
					PEX_CAUSE_REG(0),
					PEX_MASK_REG(0),
					PEX_WIN0_3_CTRL_REG(0,0),
					PEX_WIN0_3_BASE_REG(0,0),
					PEX_WIN0_3_REMAP_REG(0,0),

					PEX_WIN0_3_CTRL_REG(0,1),
					PEX_WIN0_3_BASE_REG(0,1),
					PEX_WIN0_3_REMAP_REG(0,1),

					PEX_WIN0_3_CTRL_REG(0,2),
					PEX_WIN0_3_BASE_REG(0,2),
					PEX_WIN0_3_REMAP_REG(0,2),

					PEX_WIN0_3_CTRL_REG(0,3),
					PEX_WIN0_3_BASE_REG(0,3),
					PEX_WIN0_3_REMAP_REG(0,3),

					PEX_WIN4_5_CTRL_REG(0,4),
					PEX_WIN4_5_BASE_REG(0,4),
					PEX_WIN4_5_REMAP_REG(0,4),
					PEX_WIN4_5_REMAP_HIGH_REG(0,4),

					PEX_WIN4_5_CTRL_REG(0,5),
					PEX_WIN4_5_BASE_REG(0,5),
					PEX_WIN4_5_REMAP_REG(0,5),
					PEX_WIN4_5_REMAP_HIGH_REG(0,5),

					PEX_WIN_DEFAULT_CTRL_REG(0),
					PEX_WIN_EXP_ROM_CTRL_REG(0),
					PEX_WIN_EXP_ROM_REMAP_REG(0),
					PEX_CTRL_REG(0),
					PEX_STATUS_REG(0),
					PEX_COMPLT_TMEOUT_REG(0),
					PEX_FLOW_CTRL_REG(0),
					PEX_ACK_TMR_1X_REG(0),
					PEX_TL_CTRL_REG(0),
					PEX_CFG_DIRECT_ACCESS(0,0),
					PEX_MEMORY_BAR_BASE_ADDR(0),
					PEX_MV_BAR_BASE(0),
					PEX_MV_BAR_BASE_HIGH(0),
					PEX_BAR0_INTER_REG,
					PEX_BAR0_INTER_REG_HIGH,
					PEX_BAR1_REG,
					PEX_BAR1_REG_HIGH,
					PEX_BAR2_REG,
					PEX_BAR2_REG_HIGH,
					0x00,
				};

	switch(argc)
	{
	case 2:
		reg_addr = buffaloAtoi(argv[1]);
		for(i = 0;permitted_reg[i] != 0x00; i++)
		{
			printf("reg_addr=0x%08x, permitted_reg[%d]=0x%08x\n", reg_addr, i, permitted_reg[i]);
			if(reg_addr == permitted_reg[i])
			{
				reg_val = MV_REG_READ(reg_addr);
				printf("0x%08x = 0x%08x\n", reg_addr, reg_val);
				return ;
			}
		}
		printf("not permitted to access 0x%08x\n", reg_addr);
		break;

	default:
		for(i=0; permitted_reg[i] != 0x00; i++)
		{
			reg_val = MV_REG_READ(permitted_reg[i]);
			printf("0x%08x = 0x%08x\n", permitted_reg[i], reg_val);
		}
	}
}

U_BOOT_CMD(
	pex_reg,	3,	1,	do_pex_reg_func,
	"pex_reg\t- pex register test function.\n",
);
#endif

#if defined(BUFFALO_ENABLE_GPIO_TEST)
void
do_gpio_reg_func(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	MV_U32 reg_val = 0x0;
	MV_U32 reg_addr = 0x0;
	int i = 0;

	MV_U32 permitted_reg[] = {	0x10100,
					0x10104,
					0x10108,
					0x1010c,
					0x10110,
					0x10114,
					0x10118,
					0x1011c,
					0x00,
				};

	switch(argc)
	{
	case 3:
		// write
		reg_addr = buffaloAtoi(argv[1]);
		for(i = 0;permitted_reg[i] != 0x00; i++)
		{
			if(reg_addr == permitted_reg[i])
			{
				reg_val = buffaloAtoi(argv[2]);
				MV_REG_WRITE(reg_addr, reg_val);
				return ;
			}
		}
		printf("not permitted to access 0x%08x\n", reg_addr);
		break;
	case 2:
		// read
		reg_addr = buffaloAtoi(argv[1]);
		for(i = 0;permitted_reg[i] != 0x00; i++)
		{
			printf("reg_addr=0x%08x, permitted_reg[%d]=0x%08x\n", reg_addr, i, permitted_reg[i]);
			if(reg_addr == permitted_reg[i])
			{
				reg_val = MV_REG_READ(reg_addr);
				printf("0x%08x = 0x%08x\n", reg_addr, reg_val);
				return ;
			}
		}
		printf("not permitted to access 0x%08x\n", reg_addr);
		break;
	default:
		for(i = 0;permitted_reg[i] != 0x00; i++)
		{
			reg_val = MV_REG_READ(permitted_reg[i]);
			printf("0x%08x = 0x%08x\n", permitted_reg[i], reg_val);
		}
		break;
	}

}

U_BOOT_CMD(
	gpio_reg,	3,	1,	do_gpio_reg_func,
	"gpio_reg\t- gpio register test function.\n",
);
#endif
#endif /*(CONFIG_BUFFALO_PLATFORM)*/

/*******************************************************************************
Reset environment variables.
********************************************************************************/
extern flash_info_t flash_info[];       /* info for FLASH chips */
int resetenv_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
#if defined BUFFALO_STORE_MACADDR
	char *penv = NULL;
	char *penv_old = NULL;
	env_t *env_ptr_old = NULL;
	int i = 0;
	penv_old = getenv("ethaddr");
#endif

	MV_U32 env_sec = flash_in_which_sec(&flash_info[0],
			   CFG_ENV_ADDR - flash_info[0].start[0]);

	printf("Un-Protect ENV Sector\n");

	flash_protect(FLAG_PROTECT_CLEAR,
			  CFG_ENV_ADDR,CFG_ENV_ADDR + CFG_ENV_SECT_SIZE - 1,
			  &flash_info[0]);

	
	printf("Erase sector %d ... ",env_sec);
	flash_erase(&flash_info[0], env_sec, env_sec);
	printf("done\nProtect ENV Sector\n");

	flash_protect(FLAG_PROTECT_SET,
			  CFG_ENV_ADDR,CFG_ENV_ADDR + CFG_ENV_SECT_SIZE - 1,
			  &flash_info[0]);

	printf("\nWarning: Default Environment Variables will take effect Only after RESET \n\n");

#if defined BUFFALO_STORE_MACADDR
	// store old env_ptr (usually used env_ptr)
	env_ptr_old = env_ptr;

	// create temporary env_t for default environment
	env_t DefaultEnv;
	env_ptr = (env_t *)&DefaultEnv;
	memset(env_ptr->data, (char)NULL, sizeof(env_ptr->data));

	// set default_environment variables to temporary env_t
	memcpy(env_ptr->data, default_environment, sizeof(env_ptr->data));

	// search pointer to ethaddr= on temporary env_t
	penv = strstr(env_ptr->data, "ethaddr=");
	penv += strlen("ethaddr=");

	// copy present ethaddr to temporary env_t
	for(i=0; i<strlen(ETHADDR); i++){
		penv[i] = penv_old[i];
	}

	init_board_info(1);
	env_crc_update();

	// save temporary env_t
	saveenv();
	env_ptr = env_ptr_old;
#endif //end of BUFFALO_STORE_MACADDR

	return 1;
}

U_BOOT_CMD(
        resetenv,      1,     1,      resetenv_cmd,
        "resetenv	- Return all environment variable to default.\n",
        " \n"
        "\t Erase the environemnt variable sector.\n"
);

#if CONFIG_COMMANDS & CFG_CMD_BSP
/******************************************************************************
* Category     - General
* Functionality- The commands allows the user to view the contents of the MV
*                internal registers and modify them.
* Need modifications (Yes/No) - no
*****************************************************************************/
int ir_cmd( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	MV_U32 regNum = 0x0, regVal, regValTmp, res;
	MV_8 regValBin[40];
	MV_8 cmd[30];
	int i,j = 0, flagm = 0;
	extern MV_8 console_buffer[];

	if( argc == 2 ) {
		regNum = simple_strtoul( argv[1], NULL, 16 );
	}
	else { 
		printf( "Usage:\n%s\n", cmdtp->usage );
		return 0;
	}                                                                                                        

	regVal = MV_REG_READ( regNum );
	regValTmp = regVal;
	printf( "Internal register 0x%x value : 0x%x\n ",regNum, regVal );
	printf( "\n    31      24        16         8         0" );
	printf( "\n     |       |         |         |         |\nOLD: " );

	for( i = 31 ; i >= 0 ; i-- ) {
		if( regValTmp > 0 ) {
			res = regValTmp % 2;
			regValTmp = (regValTmp - res) / 2;
			if( res == 0 )
				regValBin[i] = '0';
			else
				regValBin[i] = '1';
		}
		else
			regValBin[i] = '0';
	}

	for( i = 0 ; i < 32 ; i++ ) {
		printf( "%c", regValBin[i] );
		if( (((i+1) % 4) == 0) && (i > 1) && (i < 31) )
			printf( "-" );
	}

	readline( "\nNEW: " );
	strcpy(cmd, console_buffer);
	if( (cmd[0] == '0') && (cmd[1] == 'x') ) {
		regVal = simple_strtoul( cmd, NULL, 16 );
		flagm=1;
	}
	else {
		for( i = 0 ; i < 40 ; i++ ) {
			if(cmd[i] == '\0')
				break;
			if( i == 4 || i == 9 || i == 14 || i == 19 || i == 24 || i == 29 || i == 34 )
				continue;
			if( cmd[i] == '1' ) {
				regVal = regVal | (0x80000000 >> j);
				flagm = 1;
			}
			else if( cmd[i] == '0' ) {
				regVal = regVal & (~(0x80000000 >> j));
				flagm = 1;
			}
			j++;
		}
	}

	if( flagm == 1 ) {
		MV_REG_WRITE( regNum, regVal );
		printf( "\nNew value = 0x%x\n\n", MV_REG_READ(regNum) );
	}
	return 1;
}

U_BOOT_CMD(
	ir,      2,     1,      ir_cmd,
	"ir	- reading and changing MV internal register values.\n",
	" address\n"
	"\tDisplays the contents of the internal register in 2 forms, hex and binary.\n"
	"\tIt's possible to change the value by writing a hex value beginning with \n"
	"\t0x or by writing 0 or 1 in the required place. \n"
    	"\tPressing enter without any value keeps the value unchanged.\n"
);

/******************************************************************************
* Category     - General
* Functionality- Display the auto detect values of the TCLK and SYSCLK.
* Need modifications (Yes/No) - no
*****************************************************************************/
int clk_cmd( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    	printf( "TCLK %dMhz, SYSCLK %dMhz (UART baudrate %d)", mvTclkGet()/1000000, mvSysClkGet()/1000000, CONFIG_BAUDRATE );
	return 1;
}

U_BOOT_CMD(
	dclk,      1,     1,      clk_cmd,
	"dclk	- Display the MV device CLKs.\n",
	" \n"
	"\tDisplay the auto detect values of the TCLK and SYSCLK.\n"
);


#if defined MV_88F5181
/******************************************************************************
* Category     - Etherent
* Functionality- Display PHY ports status (using SMI access).
* Need modifications (Yes/No) - No
*****************************************************************************/
int sg_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	MV_U32 port;

	for( port = 0 ; port < mvCtrlEthMaxPortGet(); port++ ) {

		printf( "PHY %d :\n", port );
		printf( "---------\n" );

		mvEthPhyPrintStatus( mvBoardPhyAddrGet(port) );

		printf("\n");
	}

	return 1;
}

U_BOOT_CMD(
	sg,      1,     1,      sg_cmd,
	"sg	- scanning the PHYs status\n",
	" \n"
	"\tScan all the Gig port PHYs and display their Duplex, Link, Speed and AN status.\n"
);

/******************************************************************************
* Category     - DMA
* Functionality- Perform a DMA transaction
* Need modifications (Yes/No) - No
*****************************************************************************/
int mvDma_cmd( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	MV_8 cmd[20], c;
	extern MV_8 console_buffer[];
	MV_U32 chan, src, dst, byteCount, ctrlLo;
	MV_DMA_DEC_WIN win;
	MV_BOOL err;

	/* IDMA channel */
	if( argc == 2 ) 
		chan = simple_strtoul( argv[1], NULL, 16 );
	else
		chan = 0;

	/* source address */
	while(1) {
		readline( "Source Address: " );
		strcpy( cmd, console_buffer );
		src = simple_strtoul( cmd, NULL, 16 );
		if( src == 0xffffffff ) printf( "Bad address !!!\n" );
		else break;
	}

	/* desctination address */
	while(1) {
		readline( "Destination Address: " );
		strcpy(cmd, console_buffer);
		dst = simple_strtoul( cmd, NULL, 16 );
		if( dst == 0xffffffff ) printf("Bad address !!!\n");
		else break;
	}

	/* byte count */
	while(1) {
		readline( "Byte Count (up to 16M (0xffffff-1)): " );
		strcpy( cmd, console_buffer );
		byteCount = simple_strtoul( cmd, NULL, 16 );
		if( (byteCount > 0xffffff) || (byteCount == 0) ) printf("Bad value !!!\n");
		else break;
	}

	/* compose the command */
	ctrlLo = ICCLR_BLOCK_MODE | ICCLR_NON_CHAIN_MODE | ICCLR_SRC_INC | ICCLR_DST_INC;


	if (byteCount > _64K)
	{
		ctrlLo |= ICCLR_DESC_MODE_16M;
	}

	/* set data transfer limit */
	while(1) {
		printf( "Data transfer limit:\n" );
		printf( "(1) 8   bytes at a time.\n" );
		printf( "(2) 16  bytes at a time.\n" );
		printf( "(3) 32  bytes at a time.\n" );
		printf( "(4) 64  bytes at a time.\n" );
		printf( "(5) 128 bytes at a time.\n" );

		c = getc(); 
		printf( "%c\n", c );

		err = MV_FALSE;

		switch( c ) {
			case 13: /* Enter */
				ctrlLo |= (ICCLR_DST_BURST_LIM_32BYTE | ICCLR_SRC_BURST_LIM_32BYTE);
				printf( "32 bytes at a time.\n" );
				break;
			case '1':
				ctrlLo |= (ICCLR_DST_BURST_LIM_8BYTE | ICCLR_SRC_BURST_LIM_8BYTE);
				break;
			case '2':
				ctrlLo |= (ICCLR_DST_BURST_LIM_16BYTE | ICCLR_SRC_BURST_LIM_16BYTE);
				break;
			case '3':
				ctrlLo |= (ICCLR_DST_BURST_LIM_32BYTE | ICCLR_SRC_BURST_LIM_32BYTE);
				break;
			case '4':
				ctrlLo |= (ICCLR_DST_BURST_LIM_64BYTE | ICCLR_SRC_BURST_LIM_64BYTE);
				break;
			case '5':
				ctrlLo |= (ICCLR_DST_BURST_LIM_128BYTE | ICCLR_SRC_BURST_LIM_128BYTE);
				break;
			default:
				printf( "Bad value !!!\n" );
				err = MV_TRUE;
		}

		if( !err ) break;
	}

	/* set ovveride source option */
	while(1) {
		printf( "Override Source:\n" ); 
		printf( "(0) - no override\n" );
		mvDmaWinGet( 1, &win );
		printf( "(1) - use Win1 (%s)\n",mvCtrlTargetNameGet(win.target));
		mvDmaWinGet( 2, &win );
		printf( "(2) - use Win2 (%s)\n",mvCtrlTargetNameGet(win.target));
		mvDmaWinGet( 3, &win );
		printf( "(3) - use Win3 (%s)\n",mvCtrlTargetNameGet(win.target));

		c = getc(); 
		printf( "%c\n", c );

		err = MV_FALSE;

		switch( c ) {
			case 13: /* Enter */
			case '0':
				printf( "No override\n" );
				break;
			case '1':
				ctrlLo |= ICCLR_OVRRD_SRC_BAR(1);
				break;
			case '2':
				ctrlLo |= ICCLR_OVRRD_SRC_BAR(2);
				break;
			case '3':
				ctrlLo |= ICCLR_OVRRD_SRC_BAR(3);
				break;
			default:
				printf("Bad value !!!\n");
				err = MV_TRUE;
		}

		if( !err ) break;
	}

	/* set override destination option */
	while(1) {
		printf( "Override Destination:\n" ); 
		printf( "(0) - no override\n" );
		mvDmaWinGet( 1, &win );
		printf( "(1) - use Win1 (%s)\n",mvCtrlTargetNameGet(win.target));
		mvDmaWinGet( 2, &win );
		printf( "(2) - use Win2 (%s)\n",mvCtrlTargetNameGet(win.target));
		mvDmaWinGet( 3, &win );
		printf( "(3) - use Win3 (%s)\n",mvCtrlTargetNameGet(win.target));

		c = getc(); 
		printf( "%c\n", c );

		err = MV_FALSE;

	        switch( c ) {
			case 13: /* Enter */
			case '0':
				printf( "No override\n" );
				break;
			case '1':
				ctrlLo |= ICCLR_OVRRD_DST_BAR(1);
				break;
			case '2':
				ctrlLo |= ICCLR_OVRRD_DST_BAR(2);
				break;
			case '3':
				ctrlLo |= ICCLR_OVRRD_DST_BAR(3);
				break;
			default:
				printf("Bad value !!!\n");
				err = MV_TRUE;
		}

		if( !err ) break;
	}

	/* wait for previous transfer completion */
	while( mvDmaStateGet(chan) != MV_IDLE );

	/* issue the transfer */
	mvDmaCtrlLowSet( chan, ctrlLo ); 
	mvDmaTransfer( chan, src, dst, byteCount, 0 );

	/* wait for completion */
	while( mvDmaStateGet(chan) != MV_IDLE );

	printf( "Done...\n" );
	return 1;
}

U_BOOT_CMD(
	dma,      2,     1,      mvDma_cmd,
	"dma	- Perform DMA\n",
	" \n"
	"\tPerform DMA transaction with the parameters given by the user.\n"
);

#endif

/******************************************************************************
* Category     - Memory
* Functionality- Displays the MV's Memory map
* Need modifications (Yes/No) - Yes
*****************************************************************************/
int displayMemoryMap_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	mvCtrlAddrDecShow();
	return 1;
}

U_BOOT_CMD(
	map,      1,     1,      displayMemoryMap_cmd,
	"map	- Diasplay address decode windows\n",
	" \n"
	"\tDisplay controller address decode windows: CPU, PCI, Gig, DMA, XOR and COMM\n"
);



/******************************************************************************
* Category     - Memory
* Functionality- Displays the SPD information for a givven dimm
* Need modifications (Yes/No) - 
*****************************************************************************/
#include "mvDram.h"
              
int dimminfo_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
        int num = 0;
 
        if (argc > 1) {
                num = simple_strtoul (argv[1], NULL, 10);
        }
 
        printf("*********************** DIMM%d *****************************\n",num);
 
        dimmSpdPrint(num);
 
        printf("************************************************************\n");
         
        return 1;
}
 
U_BOOT_CMD(
        ddimm,      2,     1,      dimminfo_cmd,
        "ddimm  - Display SPD Dimm Info\n",
        " [0/1]\n"
        "\tDisplay Dimm 0/1 SPD information.\n"
);


/******************************************************************************
* Functionality- Go to an address and execute the code there and return,
*    defualt address is 0x40004
*****************************************************************************/
extern void cpu_dcache_flush_all(void);
extern void cpu_icache_flush_invalidate_all(void);

void mv_go(unsigned long addr,int argc, char *argv[])
{
	int rc;
	addr = MV_CACHEABLE(addr);
	char* envCacheMode = getenv("cacheMode");
 
	/*
	 * pass address parameter as argv[0] (aka command name),
	 * and all remaining args
	 */

    if(envCacheMode && (strcmp(envCacheMode,"write-through") == 0))
	{	
		int i=0;

		/* Flush Invalidate I-Cache */
		cpu_icache_flush_invalidate_all();

		/* Drain write buffer */
		asm ("mcr p15, 0, %0, c7, c10, 4": :"r" (i));

	}
	else /*"write-back"*/
	{
		int i=0;

		/* Flush Invalidate I-Cache */
		cpu_icache_flush_invalidate_all();

		/* Drain write buffer */
		asm ("mcr p15, 0, %0, c7, c10, 4": :"r" (i));

		/* Flush invalidate D-cache */
		cpu_dcache_flush_all();


    }


	rc = ((ulong (*)(int, char *[]))addr) (--argc, &argv[1]);
 
        return;
}

int g_cmd (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
        ulong   addr;

	if(!enaMonExt()){
		printf("This command can be used only if enaMonExt is set!\n");
		return 0;
	}

	addr = 0x40000;

        if (argc > 1) {
		addr = simple_strtoul(argv[1], NULL, 16);
        }
	mv_go(addr,argc,&argv[0]);
	return 1;
}                                                                                                                     

U_BOOT_CMD(
	g,      CFG_MAXARGS,     1,      g_cmd,
        "g	- start application at cached address 'addr'(default addr 0x40000)\n",
        " addr [arg ...] \n"
	"\tStart application at address 'addr'cachable!!!(default addr 0x40004/0x240004)\n"
	"\tpassing 'arg' as arguments\n"
	"\t(This command can be used only if enaMonExt is set!)\n"
);

/******************************************************************************
* Functionality- Searches for a value
*****************************************************************************/
int fi_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    MV_U32 s_address,e_address,value,i,tempData;
    MV_BOOL  error = MV_FALSE;

    if(!enaMonExt()){
	printf("This command can be used only if enaMonExt is set!\n");
	return 0;}

    if(argc == 4){
	value = simple_strtoul(argv[1], NULL, 16);
	s_address = simple_strtoul(argv[2], NULL, 16);
	e_address = simple_strtoul(argv[3], NULL, 16);
    }else{ printf ("Usage:\n%s\n", cmdtp->usage);
    	return 0;
    }     

    if(s_address == 0xffffffff || e_address == 0xffffffff) error = MV_TRUE;
    if(s_address%4 != 0 || e_address%4 != 0) error = MV_TRUE;
    if(s_address > e_address) error = MV_TRUE;
    if(error)
    {
	printf ("Usage:\n%s\n", cmdtp->usage);
        return 0;
    }
    for(i = s_address; i < e_address ; i+=4)
    {
        tempData = (*((volatile unsigned int *)i));
        if(tempData == value)
        {
            printf("Value: %x found at ",value);
            printf("address: %x\n",i);
            return 1;
        }
    }
    printf("Value not found!!\n");
    return 1;
}

U_BOOT_CMD(
	fi,      4,     1,      fi_cmd,
	"fi	- Find value in the memory.\n",
	" value start_address end_address\n"
	"\tSearch for a value 'value' in the memory from address 'start_address to\n"
	"\taddress 'end_address'.\n"
	"\t(This command can be used only if enaMonExt is set!)\n"
);

/******************************************************************************
* Functionality- Compare the memory with Value.
*****************************************************************************/
int cmpm_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    MV_U32 s_address,e_address,value,i,tempData;
    MV_BOOL  error = MV_FALSE;

    if(!enaMonExt()){
	printf("This command can be used only if enaMonExt is set!\n");
	return 0;}

    if(argc == 4){
	value = simple_strtoul(argv[1], NULL, 16);
	s_address = simple_strtoul(argv[2], NULL, 16);
	e_address = simple_strtoul(argv[3], NULL, 16);
    }else{ printf ("Usage:\n%s\n", cmdtp->usage);
    	return 0;
    }     

    if(s_address == 0xffffffff || e_address == 0xffffffff) error = MV_TRUE;
    if(s_address%4 != 0 || e_address%4 != 0) error = MV_TRUE;
    if(s_address > e_address) error = MV_TRUE;
    if(error)
    {
	printf ("Usage:\n%s\n", cmdtp->usage);
        return 0;
    }
    for(i = s_address; i < e_address ; i+=4)
    {
        tempData = (*((volatile unsigned int *)i));
        if(tempData != value)
        {
            printf("Value: %x found at address: %x\n",tempData,i);
        }
    }
    return 1;
}

U_BOOT_CMD(
	cmpm,      4,     1,      cmpm_cmd,
	"cmpm	- Compare Memory\n",
	" value start_address end_address.\n"
	"\tCompare the memory from address 'start_address to address 'end_address'.\n"
	"\twith value 'value'\n"
	"\t(This command can be used only if enaMonExt is set!)\n"
);

#if defined(MV_88F5181)

#if 0
/******************************************************************************
* Category     - Etherent
* Functionality- Display PHY ports status (using SMI access).
* Need modifications (Yes/No) - No
*****************************************************************************/
int eth_show_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{

	ethRegs();
	ethPortRegs(0);
	ethPortStatus(0);
	ethPortQueues(0,0,0,1);
	return 1;
}

U_BOOT_CMD(
	ethShow,      1,     1,      eth_show_cmd,
	"ethShow	- scanning the PHYs status\n",
	" \n"
	"\tScan all the Gig port PHYs and display their Duplex, Link, Speed and AN status.\n"
);
#endif

#include "mvEthPhy.h"

int phy_read_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    	MV_U16 phyReg;

    	mvEthPhyRegRead(simple_strtoul( argv[1], NULL, 16 ),
	                simple_strtoul( argv[2], NULL, 16), &phyReg);

	printf ("0x%x\n", phyReg);

	return 1;
}

U_BOOT_CMD(
	phyRead,      3,     3,      phy_read_cmd,
	"phyRead	- Read Phy register\n",
	" Phy_address Phy_offset. \n"
	"\tRead the Phy register. \n"
);


int phy_write_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	mvEthPhyRegWrite(simple_strtoul( argv[1], NULL, 16 ),
					 simple_strtoul( argv[2], NULL, 16 ),
					 simple_strtoul( argv[3], NULL, 16 ));

	return 1;
}

U_BOOT_CMD(
	phyWrite,      4,     4,      phy_write_cmd,
	"phyWrite	- Write Phy register\n",
	" Phy_address Phy_offset value.\n"
	"\tWrite to the Phy register.\n"
);

#endif /* #if defined(MV_88F5181) */

#endif /* MV_TINY */
